function A = dif2ss( M, C, K)

% A = dif2ss( M, C, K)
%
% Returns the A matrix defining the statespace system from
% the second order differential equation:
%
% M x''(t) + C x'(t) + K x(t) = p(t)
%
% C is optional and can be omitted for an undamped system.
%
% e.g.  dif2ss( M, [], K)
%
% The state vector for the system is a column vector with `n' displacment
% components followed by `n' velocity components.
%
% Thus, A = [       0              I     ]
%           [ - inv(M) * K   -inv(M) * C ]

if nargin < 3
	error( 'Usage: civ2ss( M, K, C).  See help for details.');
end

n = size(K,1);

if ~isempty(M) & (size(M) ~= size(K))
	error( 'Matrices M and K are not of the same dimensions');
end

if isempty(C)
	C = zeros(n);
end

if size(K) ~= size(C)
	error( 'Matrices K and C are not of the same dimensions');
end

if isempty( M)
	invM = eye(size(K));
else
	invM = inv(M);
end

A = [ zeros(n), eye(n); -invM*K, -invM*C ];

