function [vecs, vals] = difeig( M, K)

% [vecs, vals] = difeig( M, K)
%
% Returns the eigenvalues and eigenvectors for the second-degree
% differential equation
%
% M x'' + K x = 0
%
% For nxn matrices M and K, vals is an n-vector with the eigenvalues
% and vecs is an nxn matrix with mode shapes (eigenvectors) for vals(j)
% in the jth column.
%
% The order of vals is guaranteed to be from smallest (representing the
% fundamental period) to largest.  e.g.  val(n) <= val(n+1).

if nargin < 2
	error( 'Usage:  difeig( M, K)');
end

if size(M) ~= size(K)
	error( 'Matrices M and K are not of the same dimensions');
end

[vecs, vals] = eig( inv(M) * K);

% We want the eigenvalues in ascending order and the
% eigenvectors need to be sorted along with them

temp = diag(vals);
if size(temp,1) < size(temp,2)
	temp = temp';
end

% sortrows rearranges the rows of the vector
temp2 = sortrows( [ temp, vecs'], 1);

%vals = (2*pi) ./ sqrt(temp2(:,1));
vals = temp2(:,1);
vecs = temp2(:,2:(size(M,1)+1))';

