#ifndef MLF_V2
#define MLF_V2 1
#endif

#include "matlab.h"

extern mxArray * mlfMylinterp(mxArray * tx, mxArray * x, mxArray * ty);
extern void mlxMylinterp(int nlhs,
                         mxArray * plhs[],
                         int nrhs,
                         mxArray * prhs[]);

/*  A (hopefully) faster linear interpolation scheme.
 *  Points MUST be equally spaced.
 *  For now, a_ty can only be a scalar value.
 */

#define END_VEC(x) mxGetScalar(mlfEnd( x, mlfScalar(1), mlfScalar(1)))

static mxArray * Mmylinterp(int nargout_,
                            mxArray * a_tx,
                            mxArray * a_x,
                            mxArray * a_ty) {
    mxArray * y_out = mclGetUninitializedArray();

		double *tx, *x, *ty, *y, dt;
		int idx, end_x, i, length_ty, length_x;

    /* mclValidateInputs("mylinterp", 3, &tx, &x, &ty); */

		tx = mxGetPr( a_tx);
		x  = mxGetPr( a_x );
		ty = mxGetPr( a_ty);

		dt = tx[1] - tx[0];
		length_ty = END_VEC( a_ty);
		length_x  = END_VEC(a_x);

#ifdef MALLOC_VERSION
		y = (double*) malloc( sizeof(double) * length_ty);
		printf("Malloc\n");
		for( i = 0; i < length_ty; i++) {
			idx = (int)(ty[i] / dt);
#else
		printf("Non-Malloc\n");
		mlfAssign( &y_out, mlfZeros( mlfScalar(1), mlfScalar(length_ty)));
		for( i = 1; i <= length_ty; i++) {
			idx = (int)(ty[i-1] / dt);
#endif
			if( idx > length_x)
				idx = (int)  length_x - 1;
			if( idx < 0)
				idx = 0;
#ifdef MALLOC_VERSION
			y[i] = x[idx] + (ty[i] - tx[idx]) * (x[idx+1] - x[idx]) / dt;
#else
			mlfIndexAssign(&y_out, "(?)", mlfScalar(i),
					mlfScalar(x[idx] + (ty[i-1] - tx[idx]) * (x[idx+1] - x[idx]) / dt));
#endif
		}
#ifdef MALLOC_VERSION
		mlfAssign( &y_out, mlfDoubleMatrix( 1, length_ty, y, NULL));
#endif

		/* mlfAssign( &y_out, mlfScalar( x[idx] + (ty - tx[idx]) * (x[idx+1] - x[idx]) / dt)); */
    /*mclValidateOutputs("mylinterp", 1, nargout_, &y);*/

#ifdef MALLOC_VERSION
		free(y);
#endif
    return y_out;
}

/*
 * mlfMylinterp - The normal interface.  This function processes any input
 * arguments and passes them to the implementation version of the function,
 * appearing above.
 */
mxArray * mlfMylinterp(mxArray * tx, mxArray * x, mxArray * ty) {
    int nargout = 1;
    mxArray * y = mclGetUninitializedArray();
    mlfEnterNewContext(0, 3, tx, x, ty);
    y = Mmylinterp(nargout, tx, x, ty);
    mlfRestorePreviousContext(0, 3, tx, x, ty);
    return mlfReturnValue(y);
}

/*
 * mlxMylinterp - The feval interface.  The feval function calls the
 * implementation version of mylinterp through this function.
 */
void mlxMylinterp(int nlhs, mxArray * plhs[], int nrhs, mxArray * prhs[]) {
    mxArray * mprhs[3];
    mxArray * mplhs[1];
    int i;
    if (nlhs > 1) {
        mlfError(
          mxCreateString(
            "Run-time Error: File: mylinterp Line: 1 Column:"
            " 0 The function \"mylinterp\" was called with m"
            "ore than the declared number of outputs (1)"));
    }
    if (nrhs > 3) {
        mlfError(
          mxCreateString(
            "Run-time Error: File: mylinterp Line: 1 Column:"
            " 0 The function \"mylinterp\" was called with m"
            "ore than the declared number of inputs (3)"));
    }
    for (i = 0; i < 1; ++i) {
        mplhs[i] = NULL;
    }
    for (i = 0; i < 3 && i < nrhs; ++i) {
        mprhs[i] = prhs[i];
    }
    for (; i < 3; ++i) {
        mprhs[i] = NULL;
    }
    mlfEnterNewContext(0, 3, mprhs[0], mprhs[1], mprhs[2]);
    mplhs[0] = Mmylinterp(nlhs, mprhs[0], mprhs[1], mprhs[2]);
    mlfRestorePreviousContext(0, 3, mprhs[0], mprhs[1], mprhs[2]);
    plhs[0] = mplhs[0];
}

/*
 *    MEX file entry
 */


static mlfFunctionTableEntry function_table[1]
  = { { "mylinterp", mlxMylinterp, 3, 1 } };

/*
 * The function "mexFunction" is a Compiler-generated mex wrapper, suitable for
 * building a MEX-function. It initializes any persistent variables as well as
 * a function table for use by the feval function. It then calls the function
 * "mlxMylinterp". Finally, it clears the feval table and exits.
 */
void mexFunction(int nlhs, mxArray * * plhs, int nrhs, mxArray * * prhs) {
    mlfTry {
        mlfFunctionTableSetup(1, function_table);
        mclImportGlobal(0, NULL);
        mlxMylinterp(nlhs, plhs, nrhs, prhs);
        mlfFunctionTableTakedown(1, function_table);
    } mlfCatch {
        mlfFunctionTableTakedown(1, function_table);
        mclMexError();
    } mlfEndCatch
}
