function [ T, n] = spectra( x, samphz, bool_plot)
% spectra( x, samphz, plot) - plots the power spectral density of a time series
%
% Input:  x      = time series
%         samphz = sampling rate in Hertz
%         plot   = zero to disable plotting
%                  (optional -- plotting enabled if omitted)
%
% Returns the power spectral denisty in matrix form with the frequencies
% in the first column, and the spectral density in the second column

m = length(x);

n = 2;

while n < m
	n = n * 2;
end

f = fft( x, n);

% compute the power spectral density
temp = f .* conj(f) / n;
P = temp(1:n/2);

if( size(P,1) < size(P,2))
	P = P';
end

freq = samphz * (0:n/2 - 1)/ n;

if nargin > 2 & bool_plot
	plot( freq, P);
end

if nargout > 0
	T = [ freq'  P ];
end


